//
//	SDcm sample (PIC18F26J50)
//

#include "p18cxxx.h"					// Microchip C18 library
#include "sd.h"							// Private library

#define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER	

// PIC18F46J50 Family  (24/25/26/44/45/46)
     #pragma config WDTEN = OFF          //WDT disabled (enabled by SWDTEN bit)
     #pragma config PLLDIV = 2           //Divide by 2 (8 MHz internal oscillator input)
     #pragma config STVREN = ON          //stack overflow/underflow reset enabled
     #pragma config XINST = OFF          //Extended instruction set disabled
     #pragma config CPUDIV = OSC1        //No CPU system clock divide -> 48MHz
     #pragma config CP0 = OFF            //Program memory is not code-protected
     #pragma config OSC = INTOSCPLL      //Internal oscillator, PLL enabled, used by USB
     #pragma config T1DIG = ON           //Sec Osc clock source may be selected
     #pragma config LPT1OSC = OFF        //high power Timer1 mode
     #pragma config FCMEN = OFF          //Fail-Safe Clock Monitor disabled
     #pragma config IESO = OFF           //Two-Speed Start-up disabled
     #pragma config WDTPS = 32768        //1:32768
     #pragma config DSWDTOSC = INTOSCREF //DSWDT uses INTOSC/INTRC as clock
     #pragma config RTCOSC = T1OSCREF    //RTCC uses T1OSC/T1CKI as clock
     #pragma config DSBOREN = OFF        //Zero-Power BOR disabled in Deep Sleep
     #pragma config DSWDTEN = OFF        //Disabled
     #pragma config DSWDTPS = 8192       //1:8,192 (8.5 seconds)
     #pragma config IOL1WAY = OFF        //IOLOCK bit can be set and cleared
     #pragma config MSSP7B_EN = MSK7     //7 Bit address masking
     #pragma config WPFP = PAGE_1        //Write Protect Program Flash Page 0
     #pragma config WPEND = PAGE_0       //Start protection at page 0
     #pragma config WPCFG = OFF          //Write/Erase last page protect Disabled
     #pragma config WPDIS = OFF          //WPFP[5:0], WPEND, and WPCFG bits ignored  

//
// Interrupt Service Routine
//
#pragma code
#pragma interrupt HighPriorityISRCode
void HighPriorityISRCode() {
}
#pragma interruptlow LowPriorityISRCode
void LowPriorityISRCode() {
}

//
// Vector address
//
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
void _reset (void){
	    _asm goto _startup _endasm
}
#endif
#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void) {
     _asm goto HighPriorityISRCode _endasm
}
#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void) {
     _asm goto LowPriorityISRCode _endasm
}
#pragma code

//
//		M A I N
//
void main(void){
	int i;
	int fd;
	char fname[] = "test.txt";
	char buf[16] = "0123456789\r\n";

	TRISBbits.TRISB7 = 0;				// LED pin
	LATBbits.LATB7 = 1;					// on -> START

	// Mount
	if (sd_mount(0) < 0) {
		for(;;);
	}

	SD_SET_DATE(2011,3,21);
	SD_SET_TIME(12,34,56);

	// Open
	if ((fd = sd_open(fname, SD_O_CREAT|SD_O_APPEND, 0)) < 0) {
		for(;;);
	}

	// Write
	for(i = 0; i < 100; i++) {
		if (sd_write(fd, buf, 12) <= 0) {
			for(;;);
		}
	}

	// Close
	if (sd_close(fd) < 0) {
		for(;;);
	}

	// unmount
	if (sd_unmount() < 0) {
		for(;;);
	}
	LATBbits.LATB7 = 0;					// off -> OK

	for(;;);
}
