//
//  sd_config.h
//
//  SDcard control module mini - Configure file
//  Copyright (C) 2009-2011 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.12.18 v0.0  First cording
//	2010.01.11 v0.1  Support PIC18F2550
//	2010.01.16 v0.1a Split source
//	2010.02.19 v0.1b Bug fix, do not use printf()
//	2010.07.07 v0.2  Support ATmega328P
//	2011.03.21 v0.3  Support PIC26J50 SSP2 & PPS
//

#ifndef SD_CONFIG_H
#define SD_CONFIG_H


//
//	Option
//
//#define SD_OPT_BUF256					// ImageCraft compiler/PSoC limit is 256
//#define SD_OPT_STRING_H				// Use private string.h for PSoC-libc
//#define SD_OPT_CRC					// Enable CRC
//#define SD_OPT_ARGCHECK				// Logical check
//#define SD_OPT_PARTLIMCHK				// Partition border check
//#define SD_OPT_GETINF					// support sd_L1_getinfo()
#define SD_OPT_WRITE					// support sd_write()
//#define SD_OPT_UNLINK					// support sd_unlink()
//#define SD_OPT_DIR					// support sd_dir*()
//#define SD_OPT_FORCECLOSE				// Close and unmount


//
//	Device config
//

// PIC18F2550/4550: MSSP
//#define sd_L0_init()		(OpenSPI(SPI_FOSC_64, MODE_00, SMPMID),TRISCbits.TRISC6 = 0)
//#define sd_L0_term()		CloseSPI()
//#define sd_L0_send(c) 	WriteSPI(c)
//#define SD_L0_SSPBUF		SSPBUF
//#define SD_L0_SSPIF		PIR1bits.SSPIF
//#define sd_L0_cson()		(PORTCbits.RC6 = 0)
//#define sd_L0_csoff()		(PORTCbits.RC6 = 1)

// PIC18FxxJ50: MSSP2 & PPS
#include "spi.h"						// C18 library
#include "pps.h"						// C18 library
#define sd_L0_init()	{\
	PPSUnLock();\
	PPSOutput(PPS_RP17, PPS_SDO2);\
	PPSOutput(PPS_RP13, PPS_SCK2);\
	PPSInput(PPS_SDI2,  PPS_RP18);\
	PPSLock();\
	TRISCbits.TRISC6 = 0; /* SDO */ \
	TRISCbits.TRISC2 = 0; /* SCK */ \
	TRISCbits.TRISC7 = 1; /* SDI */ \
	TRISAbits.TRISA6 = 0; /* CS */ \
	OpenSPI2(SPI_FOSC_64, MODE_00, SMPMID);\
}
#define sd_L0_term()	CloseSPI2()
#define sd_L0_send(c) 	WriteSPI2(c)
#define SD_L0_SSPBUF	SSP2BUF
#define SD_L0_SSPIF		PIR3bits.SSP2IF
#define sd_L0_cson()	(LATAbits.LATA6 = 0)
#define sd_L0_csoff()	(LATAbits.LATA6 = 1)

// ATmega: SPI Clock divide
//#define SD_L1_SPICLKDIV		0					// clk(I/O)/2
//#define SD_L1_SPICLKDIV		_BV(SPR0)			// clk(I/O)/8
//#define SD_L1_SPICLKDIV		_BV(SPR1)			// clk(I/O)/32
//#define SD_L1_SPICLKDIV		_BV(SPR0)|_BV(SPR1)	// clk(I/O)/64



//
//	DEBUG
//
//#ifndef UDEBUG						// DEL 2010.02.19  Disable DEBUG
//#define printf(a,...) 				// DEL 2010.02.19
//#define i2c_lcd_position(a,b)			// DEL 2010.02.19
//#define i2c_lcd_prCString(a)			// DEL 2010.02.19
//#define i2c_lcd_prHexByte(a)			// DEL 2010.02.19
//#define i2c_lcd_prHexInt(a)			// DEL 2010.02.19
//#endif								// DEL 2010.02.19

#endif // SD_CONFIG_H
